import {take, takeUntil} from 'rxjs/operators'
import {OnDestroy, OnInit} from '@angular/core'
import {Observable, Subject} from 'rxjs'
import {ActivatedRoute, Router} from '@angular/router'
import {UrlUtils} from '../utils/url.utils'
import {ErrorHandler, ErrorModel, ErrorService} from '../service/error.service'

export abstract class SubscribingComponent implements OnInit, OnDestroy {
  protected subscribers$ = []
  protected destroy$ = new Subject<boolean>()

  protected constructor(
    protected errorService: ErrorService,
    protected activatedRoute: ActivatedRoute,
    protected router: Router,
    protected urlUtils = new UrlUtils(activatedRoute, router)
  ) {
  }

  subscribe<T>(observable: Observable<T>, resultConsumer: (T) => void, take_count = -1) {
    if (take_count >= 0) {
      observable.pipe(take(take_count), takeUntil(this.destroy$))
    } else {
      observable.pipe(takeUntil(this.destroy$))
    }

    this.subscribers$.push(observable.subscribe({
      next: resultConsumer,
      error: err => this.errorService.handleError(err)
    }))
  }

  subscribeAndNavigate(observable: Observable<any>, route: string) {
    this.subscribe(
      observable,
      () => this.urlUtils.navigateTo(route),
      1
    )
  }

  subscribeEntityById<T>(service: any, id: number, resultConsumer: (T) => void, notFoundRoute: string) {
    this.subscribers$.push(service.getById(id)
      .pipe(take(1), takeUntil(this.destroy$))
      .subscribe({
        next: e => resultConsumer(e),
        error: err => this.handleNotFoundError(err, notFoundRoute)
      }))
  }

  ngOnInit(): void {
  }

  ngOnDestroy(): void {
    this.destroy$.next(true)
    this.destroy$.complete()
  }

  handleNotFoundError(error: any, route: string) {
    if (error.status === 404) {
      this.urlUtils.navigateTo(route)
    } else {
      this.errorService.handleError(error)
    }
  }
}

export abstract class ErrorHandlingComponent extends SubscribingComponent implements ErrorHandler {
  handledErrorModels: ErrorModel[] = []

  protected constructor(
    errorService: ErrorService,
    activatedRoute: ActivatedRoute,
    router: Router
  ) {
    super(errorService, activatedRoute, router)
    this.errorService.activeErrorHandler = this
  }
}
